/**
 * Lombongo core API (for Javascript) 0.1.60.
 *
 * Copyright (c) 2023 Lombongo .
 *
 * This source code is licensed under the Apache 2.0 license found in the
 * LICENSE file in the root directory of this source tree.
 */
import { promises } from 'fs';
import { join, normalize, resolve, dirname } from 'path';
import { env } from 'process';
import fetch from 'node-fetch';

const AccountAPI = superclass =>
    class extends superclass {
        async sendMoney({ amount, accountNumber, recieverAccount, deviceFingerprint } = {}) {
            const url = `${this.baseUrl}/accounts/${recieverAccount}/send`;
            const options = {
                params: {
                    amount: amount,
                    account_number: accountNumber,
                    deviceFingerprint: deviceFingerprint
                }
            };
            const response = await this.post(url, options);
            return response;
        }

        async avatar(uid) {
            const url = `${this.baseUrl}/account/${uid}/avatar`;
            const response = await this.get(url);
            return response;
        }

        async attachCreditCard(card) {
            const url = `${this.baseUrl}/account/credit-card/attach`;
            const options = {
                params: {
                    name: card.name,
                    number: card.number,
                    exp_month: card.expMonth,
                    exp_year: card.exp_year,
                    cvc: card.cvc,
                    commission: card.commission
                }
            };
            const response = await this.post(url, options);
            return response;
        }
    };

/**
 * Yonius 0.4.9.
 *
 * Copyright (c) 2008-2020 Hive Solutions Lda.
 *
 * This source code is licensed under the Apache 2.0 license found in the
 * LICENSE file in the root directory of this source tree.
 */

class Observable {
    constructor() {
        this.callbacks = {};
    }

    bind(event, callback) {
        const callbacks = this.callbacks[event] || [];
        callbacks.push(callback);
        this.callbacks[event] = callbacks;
        return callback;
    }

    unbind(event, callback) {
        const callbacks = this.callbacks[event] || [];
        if (!callback) {
            delete this.callbacks[event];
            return;
        }

        const index = callbacks.indexOf(callback);
        if (index === -1) {
            return;
        }
        callbacks.splice(index, 1);
        this.callbacks[event] = callbacks;
    }

    trigger(event) {
        const callbacks = this.callbacks[event] || [];
        const results = [];
        for (const callback of callbacks) {
            const result = callback.apply(this, Array.prototype.slice.call(arguments, 1));
            result !== undefined && result !== null && results.push(result);
        }
        return Promise.all(results);
    }
}

const verify = function(
    condition,
    message = null,
    code = null,
    exception = null,
    kwargs = {}
) {
    if (condition) return;
    message = message || "Verification failed";
    const Exception = exception || Error;
    kwargs = Object.assign({}, kwargs);
    if (message !== null && message !== undefined) kwargs.message = message;
    if (code !== null && message !== undefined) kwargs.code = code;
    const throwable = new Exception(kwargs.message || undefined);
    throwable.kwargs = kwargs;
    for (const [key, value] of Object.entries(kwargs)) {
        if (throwable[key] !== undefined) continue;
        throwable[key] = value;
    }
    throw throwable;
};

let HOME_DIR = null;

const pathExists = async function(path) {
    try {
        await promises.access(path);
    } catch (error) {
        return false;
    }
    return true;
};

const expandUser = function(path) {
    if (!path) return path;
    if (path === "~") return _homeDir();
    if (path.slice(0, 2) !== "~/") return path;
    return join(HOME_DIR, path.slice(2));
};

const getEnv = function(name) {
    // eslint-disable-next-line no-undef
    if (typeof Deno !== "undefined") return Deno.env.get(name);
    return env[name];
};

const getEnvObject = function() {
    // eslint-disable-next-line no-undef
    if (typeof Deno !== "undefined") return Deno.env.toObject();
    return env;
};

const _homeDir = function() {
    if (HOME_DIR !== null) return HOME_DIR;
    const isWindows = Boolean(typeof process !== "undefined" && process.platform === "win32");
    HOME_DIR = getEnv(isWindows ? "USERPROFILE" : "HOME") || "/";
    return HOME_DIR;
};

const FILE_NAME = "yonius.json";

const HOME_FILE = "~/.home";

const IMPORT_NAMES = ["$import", "$include", "$IMPORT", "$INCLUDE"];

const globals =
    typeof global === "undefined"
        ? typeof window === "undefined"
            ? typeof self === "undefined"
                ? {}
                : self
            : window
        : global;

globals.CONFIGS = globals.CONFIGS === undefined ? {} : globals.CONFIGS;

globals.CONFIG_F = globals.CONFIG_F === undefined ? [] : globals.CONFIG_F;

globals.HOMES = globals.HOMES === undefined ? [] : globals.HOMES;

globals.LOADED = globals.LOADED === undefined ? false : globals.LOADED;

const load = async function(
    names = [FILE_NAME],
    path = null,
    encoding = "utf-8",
    force = false,
    ctx = null
) {
    if (globals.LOADED && !force) return;
    let paths = [];
    const homes = await getHomes();
    for (const home of homes) {
        paths = paths.concat([join(home), join(home, ".config")]);
    }
    if (path) paths.push(path);
    for (const path of paths) {
        for (const name of names) {
            await loadFile(name, path, encoding, ctx);
        }
    }
    await loadEnv(ctx);
    globals.LOADED = true;
};

const loadFile = async function(
    name = FILE_NAME,
    path = null,
    encoding = "utf-8",
    ctx = null
) {
    const configs = ctx ? ctx.configs : globals.CONFIGS;
    const configF = ctx ? ctx.configF : globals.CONFIG_F;

    let key;
    let value;
    let exists;
    let filePath;

    if (path) path = normalize(path);
    if (path) filePath = join(path, name);
    else filePath = name;

    filePath = resolve(filePath);
    filePath = normalize(filePath);
    const basePath = dirname(filePath);

    exists = await pathExists(filePath);
    if (!exists) return;

    exists = configF.includes(filePath);
    if (exists) configF.splice(configF.indexOf(filePath), 1);
    configF.push(filePath);

    const data = await promises.readFile(filePath, { encoding: encoding });
    const dataJ = JSON.parse(data);

    await _loadIncludes(basePath, dataJ, encoding);

    for ([key, value] of Object.entries(dataJ)) {
        if (!_isValid(key)) continue;
        configs[key] = value;
    }
};

const loadEnv = async function(ctx = null) {
    const env = getEnvObject();
    const configs = ctx ? ctx.configs : globals.CONFIGS;
    if (env === undefined || env === null) return;
    Object.entries(env).forEach(function([key, value]) {
        configs[key] = value;
    });
};

const getHomes = async function(
    filePath = HOME_FILE,
    fallback = "~",
    encoding = "utf-8",
    forceDefault = false
) {
    if (globals.HOMES.length > 0) return globals.HOMES;

    const env = getEnvObject();

    globals.HOMES = env.HOMES === undefined ? null : env.HOMES;
    globals.HOMES = globals.HOMES ? globals.HOMES.split(";") : globals.HOMES;
    if (globals.HOMES !== null) return globals.HOMES;

    fallback = expandUser(fallback);
    fallback = normalize(fallback);
    globals.HOMES = [fallback];

    filePath = expandUser(filePath);
    filePath = normalize(filePath);
    const exists = await pathExists(filePath);
    if (!exists) return globals.HOMES;

    if (!forceDefault) globals.HOMES.splice(0, globals.HOMES.length);

    let data = await promises.readFile(filePath, { encoding: encoding });
    data = data.trim();

    let paths = data.split(/\r?\n/);
    paths = paths.map(v => v.trim());

    for (let path of paths) {
        path = path.trim();
        if (!path) continue;
        path = expandUser(path);
        path = normalize(path);
        globals.HOMES.push(path);
    }

    return globals.HOMES;
};

const _loadIncludes = async function(basePath, config, encoding = "utf-8") {
    let includes = [];

    for (const alias of IMPORT_NAMES) {
        includes = config[alias] === undefined ? includes : config[alias];
    }

    if (typeof includes === "string") {
        includes = includes.split(";");
    }

    for (const include of includes) {
        await loadFile(include, basePath, encoding);
    }
};

const _isValid = function(key) {
    if (IMPORT_NAMES.includes(key)) return false;
    return true;
};

class MixinBuilder {
    constructor(superclass) {
        this.superclass = superclass;
    }

    with(...mixins) {
        return mixins.reduce((c, mixin) => mixin(c), this.superclass);
    }
}

const mix = superclass => new MixinBuilder(superclass);

/**
 * Encodes the multiple values as and encoded URI component, the
 * values can be wither defined as an array (order is preserved)
 * or as an object (where sequence order is not preserved).
 *
 * The value of each item can be either a primitive type or a sequence
 * in case it's of sequence the values are going to be encoded as
 * multiple parameters separated by the '&' character.
 *
 * @param {(Array|Object[])} values The values to be encoded as an
 * URI component (like GET params).
 * @returns {String} A string with the query encoded values.
 */
const urlEncode = function(values) {
    // constructs the parts array that is going to
    // store the multiple and values
    const parts = [];

    // in case the provided value is not an array
    // then assumes it's an object and retrieve entries
    if (!Array.isArray(values)) {
        values = Object.entries(values);
    }

    // iterates over the complete set of pairs available
    // from the key value pairs to be able to encode them
    // properly, notice that the values themselves can be
    // sequences allowing multiple repetition of key
    values.forEach(([key, value]) => {
        if (!Array.isArray(value)) {
            value = [value];
        }
        const keyEncoded = encodeURIComponent(key);
        value.forEach(_value => {
            if (_value === undefined || _value === null) {
                return;
            }
            const valueEncoded = encodeURIComponent(_value);
            parts.push(`${keyEncoded}=${valueEncoded}`);
        });
    });

    // joins the complete set of parts with the and
    // separator and then returns the final string value
    return parts.join("&");
};

const AUTH_ERRORS = [401, 403, 440, 499];

class API$1 extends Observable {
    constructor(kwargs = {}) {
        super();
        this.kwargs = kwargs;
    }

    async build(method, url, options = {}) {}

    async authCallback(params, headers) {}

    async get(url, options = {}) {
        const result = await this.methodBasic("GET", url, options);
        return result;
    }

    async post(url, options = {}) {
        const result = await this.methodPayload("POST", url, options);
        return result;
    }

    async put(url, options = {}) {
        const result = await this.methodPayload("PUT", url, options);
        return result;
    }

    async delete(url, options = {}) {
        const result = await this.methodBasic("DELETE", url, options);
        return result;
    }

    async patch(url, options = {}) {
        const result = await this.methodPayload("PATCH", url, options);
        return result;
    }

    async options(url, options = {}) {
        const result = await this.methodBasic("OPTIONS", url, options);
        return result;
    }

    async methodBasic(method, url, options = {}) {
        options.params = options.params !== undefined ? options.params : {};
        options.headers = options.headers !== undefined ? options.headers : {};
        try {
            return await this._methodBasic(method, url, options);
        } catch (err) {
            if (AUTH_ERRORS.includes(err.code)) {
                await this.authCallback(options.params, options.headers);
                return await this._methodBasic(method, url, options);
            } else {
                throw err;
            }
        }
    }

    async methodPayload(method, url, options = {}) {
        options.params = options.params !== undefined ? options.params : {};
        options.headers = options.headers !== undefined ? options.headers : {};
        try {
            return await this._methodPayload(method, url, options);
        } catch (err) {
            if (AUTH_ERRORS.includes(err.code)) {
                await this.authCallback(options.params, options.headers);
                return await this._methodPayload(method, url, options);
            } else {
                throw err;
            }
        }
    }

    async _methodBasic(method, url, options = {}) {
        const params = options.params !== undefined ? options.params : {};
        const headers = options.headers !== undefined ? options.headers : {};
        const kwargs = options.kwargs !== undefined ? options.kwargs : {};
        const handle = options.handle !== undefined ? options.handle : true;
        await this.build(method, url, {
            params: params,
            headers: headers,
            kwargs: kwargs
        });
        const query = urlEncode(params || {});
        if (query) url += url.includes("?") ? "&" + query : "?" + query;
        const response = await fetch(url, {
            method: method,
            headers: headers || {}
        });
        const result = handle ? await this._handleResponse(response) : response;
        return result;
    }

    async _methodPayload(method, url, options = {}) {
        const params = options.params !== undefined ? options.params : {};
        let headers = options.headers !== undefined ? options.headers : {};
        let data = options.data !== undefined ? options.data : null;
        const dataJ = options.dataJ !== undefined ? options.dataJ : null;
        const dataM = options.dataM !== undefined ? options.dataM : null;
        let mime = options.mime !== undefined ? options.mime : null;
        const kwargs = options.kwargs !== undefined ? options.kwargs : {};
        const handle = options.handle !== undefined ? options.handle : true;

        await this.build(method, url, {
            params: params,
            headers: headers,
            data: data,
            dataJ: dataJ,
            dataM: dataM,
            mime: mime,
            kwargs: kwargs
        });

        const query = urlEncode(params || {});

        if (data !== null) {
            if (query) url += url.includes("?") ? "&" + query : "?" + query;
        } else if (dataJ !== null) {
            data = JSON.stringify(dataJ);
            if (query) url += url.includes("?") ? "&" + query : "?" + query;
            mime = mime || "application/json";
        } else if (dataM !== null) {
            if (query) url += url.includes("?") ? "&" + query : "?" + query;
        } else if (query) {
            data = query;
            mime = mime || "application/x-www-form-urlencoded";
        }

        headers = Object.assign({}, headers);
        if (mime) headers["Content-Type"] = mime;

        const response = await fetch(url, {
            method: method,
            headers: headers || {},
            body: data
        });
        const result = handle ? await this._handleResponse(response) : response;
        return result;
    }

    async _handleResponse(response, errorMessage = "Problem in request") {
        let result = null;
        if (response.headers.get("content-type").toLowerCase().startsWith("application/json")) {
            result = await response.json();
        } else {
            result = await response.blob();
        }
        verify(response.ok, result.error || errorMessage, response.status || 500);
        return result;
    }
}

const load$1 = async function() {
    await load();
};

const currenciesAPI = superclass =>
    class extends superclass {
        async getCurrencies() {
            const url = `${this.baseUrl}/currencies`;
            const response = await this.get(url);
            return response;
        }
    };

const CurrentCommissionAPI = superclass =>
    class extends superclass {
        async getCurrentCommission() {
            const url = `${this.baseUrl}/current-commission`;
            const response = await this.get(url);
            return response;
        }
    };

const fxRatesAPI = superclass =>
    class extends superclass {
        async getFxRates() {
            const url = `${this.baseUrl}/fx-rates`;
            const response = await this.get(url);
            return response;
        }

        async getFxRate(originalCurrency, finalCurrency) {
            const url = `${this.baseUrl}/fx-rate`;
            const options = {
                params: { originalCurrency: originalCurrency, finalCurrency: finalCurrency }
            };
            const response = await this.post(url, options);
            return response;
        }
    };

const PaymentAPI = superclass =>
    class extends superclass {
        async mbway(amount, documentID, email, phoneNumber) {
            const url = `${this.baseUrl}/payment/mbway`;
            const options = {
                params: {
                    amount: amount,
                    documentID: documentID,
                    email: email,
                    phoneNumber: phoneNumber
                }
            };
            const response = await this.post(url, options);
            return response;
        }

        async mbwayStatus(idPayment) {
            const url = `${this.baseUrl}/payment/mbway/status`;
            const options = { params: { mbway_id: idPayment } };
            const response = await this.post(url, options);
            return response;
        }

        async mcx(amount, documentID, phoneNumber) {
            const url = `${this.baseUrl}/payment/mcx`;
            const options = {
                params: {
                    amount: amount,
                    documentID: documentID,
                    phoneNumber: phoneNumber
                }
            };
            const response = await this.post(url, options);
            return response;
        }
    };

const TransactionsAPI = superclass =>
    class extends superclass {
        async getFXConfig(originalCurrency, finalCurrency, amount) {
            const url = `${this.baseUrl}/transactions/sepa/config`;
            const options = {
                params: {
                    originalCurrency: originalCurrency,
                    finalCurrency: finalCurrency,
                    amount: amount
                }
            };
            const response = await this.post(url, options);
            return response;
        }

        async transferCrossCurrency({
            sepaBeneficiaryIban,
            sepaBeneficiaryName,
            sepaBeneficiarySwift,
            originalAmount,
            originalCurrency,
            finalAmount,
            finalCurrency,
            fxRate,
            commission
        } = {}) {
            const url = `${this.baseUrl}/users/send/cross-currency`;
            const options = {
                params: {
                    sepaBeneficiaryIban: sepaBeneficiaryIban,
                    sepaBeneficiaryName: sepaBeneficiaryName,
                    sepaBeneficiarySwift: sepaBeneficiarySwift,
                    originalAmount: originalAmount,
                    originalCurrency: originalCurrency,
                    finalAmount: finalAmount,
                    finalCurrency: finalCurrency,
                    fxRate: fxRate,
                    commission: commission
                }
            };
            const response = await this.post(url, options);
            return response;
        }
    };

const UsersAPI = superclass =>
    class extends superclass {
        async createUser(user, deviceFingerprint) {
            const url = `${this.baseUrl}/users`;
            const options = { params: user, deviceFingerprint: deviceFingerprint };
            const response = await this.post(url, options);
            return response;
        }

        async getMyAccounts() {
            const url = `${this.baseUrl}/users/me/accounts`;
            const response = await this.get(url);
            return response;
        }

        async getMyTransactions() {
            const url = `${this.baseUrl}/users/me/transactions`;
            const response = await this.get(url);
            return response;
        }

        async setAvatar(avatarUrl) {
            const url = `${this.baseUrl}/users/avatar`;
            const options = { params: { avatarUrl: avatarUrl } };
            const response = await this.post(url, options);
            return response;
        }

        async getAvatar() {
            const url = `${this.baseUrl}/users/me/avatar`;
            const response = await this.get(url);
            return response;
        }

        async getAllUsers() {
            const url = `${this.baseUrl}/users`;
            const response = await this.get(url);
            return response;
        }

        async searchUser(q) {
            const url = `${this.baseUrl}/users/search?q=${q}`;
            const response = await this.get(url);
            return response;
        }

        async setUserFCMToken(token) {
            const url = `${this.baseUrl}/users/firebase-fcm-token`;
            const options = { params: { token: token } };
            const response = await this.post(url, options);
            return response;
        }
    };

const CardsAPI = superclass =>
    class extends superclass {
        async listCards() {
            const url = `${this.baseUrl}/cards/me`;
            const response = await this.get(url);
            return response;
        }
        async attachCard(token, cardDetails) {
            const url = `${this.baseUrl}/users/attach-card`;
            const options = {
                params: {
                    token: token,
                    cardDetails: cardDetails,
                }
            };
            const response = await this.post(url, options);
            return response;
        }
    };

const PaymentMethodsAPI = superclass =>
    class extends superclass {
        async getPaymentMethodsByCurrency(currencyId) {
            const url = `${this.baseUrl}/payment-methods/${currencyId}`;
            const response = await this.get(url);
            return response;
        }
    };

class API extends mix(API$1).with(
    AccountAPI,
    CardsAPI,
    CurrentCommissionAPI,
    fxRatesAPI,
    PaymentAPI,
    PaymentMethodsAPI,
    TransactionsAPI,
    UsersAPI,
    currenciesAPI
) {
    constructor(kwargs = {}) {
        super(kwargs);
        this.baseUrl = kwargs.baseUrl;
        this.token = kwargs.token === undefined ? this.token : kwargs.token;
    }

    static async load() {
        await load$1();
    }

    async build(method, url, options = {}) {
        await super.build(method, url, options);
        options.headers = options.params !== undefined ? options.headers : {};
        options.kwargs = options.kwargs !== undefined ? options.kwargs : {};
        delete options.kwargs.auth;
        options.headers.token = this.token;
    }
}

export { API, AccountAPI, PaymentAPI, TransactionsAPI, UsersAPI };
//# sourceMappingURL=lombongo-core-api.esm.js.map
